@@ -164,7 +164,7 @@ main (int argc, char **argv)
                                              NULL, NULL, NULL, false);
   gl_list_t arg_line = gl_list_create_empty (GL_LINKED_LIST,
                                              NULL, NULL, NULL, false);
-  size_t i, line;
+  size_t i, line = 1;
   Buffer *scratch_bp;
   astr as;
   bool ok = true;
@@ -264,8 +264,8 @@ main (int argc, char **argv)
               gl_list_add_last (arg_type, (void *) ARG_FILE);
               gl_list_add_last (arg_arg, (void *) optarg);
               gl_list_add_last (arg_line, (void *) line);
+              line = 1;
             }
-          line = 1;
           break;
         }
     }
@@ -318,9 +318,9 @@ main (int argc, char **argv)
       switch ((int) gl_list_get_at (arg_type, i))
         {
         case ARG_FUNCTION:
-          ok = get_function (arg);
+          ok = get_function (arg) != NULL;
           if (ok)
-            ok = execute_function (arg, 1);
+            ok = execute_function (arg, 1) != leNIL;
           else
             minibuf_error ("Function `%s' not defined", arg);
           break;
@@ -333,7 +333,7 @@ main (int argc, char **argv)
           {
             ok = find_file (arg);
             if (ok)
-              FUNCALL_ARG (goto_line, line);
+              FUNCALL_ARG (goto_line, (size_t) gl_list_get_at (arg_line, i));
             lastflag |= FLAG_NEED_RESYNC;
           }
           break;
