@@ -311,7 +311,7 @@ draw_status_line (size_t line, Window * wp)
   term_move (line, 0);
   bs = astr_afmt (astr_new (), "(%d,%d)", pt.n + 1,
                   get_goalc_bp (get_window_bp (wp), window_pt (wp)));
-  as = astr_afmt (astr_new (), "--%s%2s  %-15s   %s %-9s (Text",
+  as = astr_afmt (astr_new (), "--%s%2s  %-15s   %s %-9s (Fundamental",
                   eol_type, make_mode_line_flags (wp), get_buffer_name (get_window_bp (wp)),
                   make_screen_pos (wp, &buf), astr_cstr (bs));
   free (buf);
