@@ -46,7 +46,7 @@ struct Undo
 
   /* Flag indicating that reverting this undo leaves the buffer
      in an unchanged state. */
-  int unchanged;
+  bool unchanged;
 
   /* The block to insert. */
   struct
@@ -72,13 +72,13 @@ undo_save (int type, Point pt, size_t osize, size_t size)
 {
   Undo *up;
 
-  if (cur_bp->flags & BFLAG_NOUNDO || undo_nosave)
+  if (get_buffer_noundo (cur_bp) || undo_nosave)
     return;
 
   up = (Undo *) XZALLOC (Undo);
   up->type = type;
   up->pt = pt;
-  if (!(cur_bp->flags & BFLAG_MODIFIED))
+  if (!get_buffer_modified (cur_bp))
     up->unchanged = true;
 
   if (type == UNDO_REPLACE_BLOCK)
@@ -88,11 +88,11 @@ undo_save (int type, Point pt, size_t osize, size_t size)
       up->block.text = copy_text_block (pt.n, pt.o, osize);
     }
 
-  up->next = cur_bp->last_undop;
-  cur_bp->last_undop = up;
+  up->next = get_buffer_last_undop (cur_bp);
+  set_buffer_last_undop (cur_bp, up);
 
   if (!doing_undo)
-    cur_bp->next_undop = up;
+    set_buffer_next_undop (cur_bp, up);
 }
 
 /*
@@ -131,10 +131,8 @@ revert_action (Undo * up)
 
   doing_undo = false;
 
-  /* If reverting this undo action leaves the buffer unchanged,
-     unset the modified flag. */
   if (up->unchanged)
-    cur_bp->flags &= ~BFLAG_MODIFIED;
+    set_buffer_modified (cur_bp, false);
 
   return up->next;
 }
@@ -145,7 +143,7 @@ Undo some previous changes.
 Repeat this command to undo more changes.
 +*/
 {
-  if (cur_bp->flags & BFLAG_NOUNDO)
+  if (get_buffer_noundo (cur_bp))
     {
       minibuf_error ("Undo disabled in this buffer");
       return leNIL;
@@ -154,14 +152,14 @@ Repeat this command to undo more changes.
   if (warn_if_readonly_buffer ())
     return leNIL;
 
-  if (cur_bp->next_undop == NULL)
+  if (get_buffer_next_undop (cur_bp) == NULL)
     {
       minibuf_error ("No further undo information");
-      cur_bp->next_undop = cur_bp->last_undop;
+      set_buffer_next_undop (cur_bp, get_buffer_last_undop (cur_bp));
       return leNIL;
     }
 
-  cur_bp->next_undop = revert_action (cur_bp->next_undop);
+  set_buffer_next_undop (cur_bp, revert_action (get_buffer_next_undop (cur_bp)));
   minibuf_write ("Undo!");
 }
 END_DEFUN
@@ -180,14 +178,11 @@ free_undo (Undo *up)
 }
 
 /*
- * Set unchanged flags to false except for the last undo action (the
- * argument), which is set to true.
+ * Set unchanged flags to false.
  */
 void
 undo_set_unchanged (Undo *up)
 {
-  if (up)
-    up->unchanged = true;
-  for (up = up->next; up; up = up->next)
+  for (; up; up = up->next)
     up->unchanged = false;
 }
